@@ -43,8 +43,6 @@
 
 #ifdef USE_CONIC
 #include <conic/conic.h>
-#define USER_DATA_MAGIC 0xacdcacdc
-static ConIcConnection *connection;
 #endif
 
 static GMainLoop *gsloop = NULL;
@@ -54,13 +52,30 @@ static gboolean can_connect = TRUE;
 static gboolean is_connecting = FALSE;
 
 #ifdef USE_CONIC
+static ConIcConnection *connection;
+static GSList *connections = NULL;
+static int connection_count = -1;        /* negative for the initial state */
+
+#define USER_DATA_MAGIC 0xacdcacdc
 #define OSSO_CON_IC_CONNECTING             0x05
 
+static gint g_mozilla_iap_strcmp(gconstpointer a, gconstpointer b)
+{
+    return strcmp(a, b);
+}
+
+static void g_mozilla_iap_free(gpointer data, gpointer u)
+{
+    g_free(data);
+}
+
 static void connection_cb(ConIcConnection *connection,
                           ConIcConnectionEvent *event,
                           gpointer user_data)
 {
     const gchar *iap_id, *bearer;
+    gchar *tmp_iap;
+    GSList *tmp_list;
     ConIcConnectionStatus status;
     ConIcConnectionError error;
 
@@ -72,27 +87,113 @@ static void connection_cb(ConIcConnection *connection,
     iap_id = con_ic_event_get_iap_id(CON_IC_EVENT(event));
     bearer = con_ic_event_get_bearer_type(CON_IC_EVENT(event));
 
+    g_debug("connection_cb(%p, %s, %s, %i, %i) count %i",
+            (void *)connection, iap_id, bearer, status, error,
+            connection_count);
+    
+    if (connection_count < 0) {
+        connection_count = 0;
+        /*
+         * If it is the initial connectivity signal and we are not
+         * connected, don't change the internal state.
+         */
+        if (iap_id == NULL)
+            return;
+    }
+    
     switch (status) {
+    case CON_IC_STATUS_NETWORK_UP:
+        /*
+         * We have a network connection, but the IAP is not yet fully
+         * connected. Pretend we are already connected in order to let
+         * Connectivity invoke the browser in this state. We will get
+         * a connected signal later when everything is ok.
+         */
+        status = CON_IC_STATUS_CONNECTED;
+        /* fall through */
+    case CON_IC_STATUS_CONNECTED:
+        /*
+         * If we are not already tracking the status of this iap,
+         * i.e., we haven't received a CONNECTED signal about it, add
+         * it to the list of tracked iaps. We may get multiple
+         * CONNECTED signals for the same iap if we requested the
+         * connection ourselves.
+         */
+        if (g_slist_find_custom(connections, iap_id,
+                                g_mozilla_iap_strcmp) == NULL) {
+            g_debug("connection_cb(%p, %s, %s, %i, %i) connected",
+                    (void *)connection, iap_id, bearer, status, error);
+            tmp_iap = g_strdup(iap_id);
+            if (tmp_iap == NULL) {
+                g_debug("connection_cb(): g_strdup out of memory!");
+                /* XXX signal out of memory error */
+                return;
+            }
+            tmp_list = g_slist_prepend(connections, tmp_iap);
+            if (tmp_list == NULL) {
+                g_debug("connection_cb(): g_slist_prepend out of memory!");
+                g_free(tmp_iap);
+                /* XXX signal out of memory error */
+                return;
+            }
+            connections = tmp_list;
+            connection_count++;
+        } else {
+            /*
+             * We have already received the CONNECTED signal for this
+             * iap, no need to do anything.
+             */
+            g_debug("connection_cb(%p, %s, %s, %i, %i) already connected",
+                    (void *)connection, iap_id, bearer, status, error);
+            return;
+        }
+        connected = TRUE;
+        is_connecting = FALSE;
+        g_signal_emit_by_name(G_OBJECT(webglobal),
+                              "connectivity-status", status);
+        return;
 
-        case CON_IC_STATUS_CONNECTED:
-            connected = TRUE;
-            is_connecting = FALSE;
-            g_signal_emit_by_name(G_OBJECT(webglobal), "connectivity-status", status);
-            return ;
-
-        case CON_IC_STATUS_DISCONNECTED:
+    case CON_IC_STATUS_DISCONNECTED:
+        /*
+         * See if we are already tracking this iap.
+         */
+        if (iap_id != NULL &&
+            (tmp_list = g_slist_find_custom(connections, iap_id,
+                                            g_mozilla_iap_strcmp)) != NULL) {
+            g_debug("connection_cb(%p, %s, %s, %i, %i) disconnected",
+                    (void *)connection, iap_id, bearer, status, error);
+            tmp_iap = tmp_list->data;
+            connections = g_slist_remove(connections, tmp_iap);
+            g_free(tmp_iap);
+            --connection_count;
+        }
 #ifdef USE_DBUS
         // return;
 #else
         break;
 #endif
-        case CON_IC_STATUS_DISCONNECTING:
-            break;
+    case CON_IC_STATUS_DISCONNECTING:
+        break;
 
-        default:
-            break;
+    default:
+        break;
+    }
+    
+    /*
+     * If we are still connected (e.g. in the change connection use
+     * case we might get a connect signal first and a disconnected
+     * signal later), don't signal disconnection.
+     */
+    if (connection_count > 0) {
+        g_debug("connection_cb(%p, %s, %s, %i, %i) still connected (%d)",
+                (void *)connection, iap_id, bearer, status, error,
+                connection_count);
+        return;
     }
 
+    g_debug("connection_cb(%p, %s, %s, %i, %i) not connected (%d)",
+            (void *)connection, iap_id, bearer, status, error,
+            connection_count);
     connected = FALSE;
     g_signal_emit_by_name(G_OBJECT(webglobal), "connectivity-status", status);
     if (gsloop && g_main_loop_is_running(gsloop))
@@ -108,6 +209,10 @@ void g_mozilla_connectivity_handle (GMozillaEngine *self, gint event)
     switch (event) {
 #ifdef USE_CONIC
     case CON_IC_STATUS_CONNECTED:
+
+        gtk_moz_embed_common_observe("@mozilla.org/network/io-service;1", NULL, "profile-change-net-teardown", NULL);
+        gtk_moz_embed_common_observe("@mozilla.org/network/io-service;1", NULL, "profile-change-net-restore", NULL);
+
         ULOG_INFO_F("CON_IC_STATUS_CONNECTED: self:%p, global:%p\n", self, webglobal);
 
         // fix for bug 55796
@@ -134,7 +239,7 @@ void g_mozilla_connectivity_handle (GMozillaEngine *self, gint event)
             }
               /*else if (self->is_download)
                 g_signal_emit_by_name(G_OBJECT(self), "finished_loading", 0, self->last_net_status, 15000, TRUE);*/
-        } 
+        }
         if (gsloop && g_main_loop_is_running(gsloop))
             g_main_loop_quit (gsloop);
         gsloop = NULL;
@@ -167,13 +272,14 @@ void g_mozilla_connectivity_handle (GMozillaEngine *self, gint event)
 #endif
     }
     can_connect = FALSE;
-    if (self && self->waiting_connection) {
-        g_signal_emit_by_name(G_OBJECT(self), "finished_loading", 0, self->last_net_status, self->page_size, TRUE);
-        self->progress_bar_state = -1;
-        self->user_iteraction_happens = FALSE;
-    }
-
+    if(self) {
+        if (self->waiting_connection) {
+            g_signal_emit_by_name(G_OBJECT(self), "finished_loading", 0, self->last_net_status, self->page_size, TRUE);
+            self->progress_bar_state = -1;
+            self->user_iteraction_happens = FALSE;
+        }
     self->waiting_connection = FALSE;
+    }
     can_connect = TRUE;
 }
 
@@ -240,6 +346,28 @@ connectivity_cb (DBusConnection *connection,
 }
 #endif
 
+static void
+g_mozilla_connectivity_conf_handler_func (GConfClient * client, guint cnxn_id, GConfEntry * entry, GMozillaWeb *self)
+{
+   g_debug("Changed GConf proxy settings\n");
+   g_timeout_add(1000, g_mozilla_connectivity_set_proxy, self);
+/*
+   if (self)
+   {
+      GConfValue*  val = gconf_entry_get_value (entry);
+      if (val && val->type == GCONF_VALUE_BOOL)
+      {
+        g_debug(">>>>>>Func:%s::%d, val:%i\n", __PRETTY_FUNCTION__, __LINE__, gconf_value_get_bool (val));
+      }
+
+      if (val && val->type == GCONF_VALUE_INT)
+        g_debug(">>>>>>Func:%s::%d, val:%i\n", __PRETTY_FUNCTION__, __LINE__, gconf_value_get_int (val));
+      if (val && val->type == GCONF_VALUE_STRING)
+        g_debug(">>>>>>Func:%s::%d, val:%s\n", __PRETTY_FUNCTION__, __LINE__, gconf_value_get_string (val));
+  }
+*/
+}
+
 void
 g_mozilla_connectivity_set_global (GMozillaWeb *self)
 {
@@ -249,9 +377,16 @@ g_mozilla_connectivity_set_global (GMozillaWeb *self)
     connection = con_ic_connection_new();
     g_assert(connection != NULL);
 
+    /* Make sure we are in the initial state */
+    connection_count = -1;
+    
     g_signal_connect(G_OBJECT(connection), "connection-event",
                      G_CALLBACK(connection_cb),
                      GINT_TO_POINTER(USER_DATA_MAGIC));
+
+    /* Receive notification about all connectivity events */
+    g_object_set(G_OBJECT(connection),
+                 "automatic-connection-events", TRUE, NULL);
 #endif
 #ifdef USE_DBUS
     DBusConnection *sys_conn;
@@ -267,6 +402,10 @@ g_mozilla_connectivity_set_global (GMozillaWeb *self)
     TRACE_LOG("filter: %s", filter);
     dbus_bus_add_match (sys_conn, filter, NULL);
 #endif /* USE_DBUS */
+    const char *val = "/system/http_proxy/use_http_proxy";
+    GConfClient *client = gconf_client_get_default();
+    gconf_client_add_dir(client, val, GCONF_CLIENT_PRELOAD_NONE, NULL);
+    gconf_client_notify_add(client, val, g_mozilla_connectivity_conf_handler_func, self, NULL, NULL);
 }
 
 void
@@ -281,10 +420,23 @@ g_mozilla_connectivity_unset_global (GMozillaWeb *self)
     g_snprintf(filter, sizeof (filter),
                "type='signal',interface='%s'",
                ICD_DBUS_INTERFACE);
-#endif /* USE_DBUS */
+#endif /* USE_CONIC */
     dbus_bus_remove_match(sys_conn, filter, NULL);
     dbus_connection_remove_filter(sys_conn, connectivity_cb, self);
 #endif /* USE_DBUS */
+#ifdef USE_CONIC
+    /* Stop receiving automatic events and free the connection object */
+    g_object_set(G_OBJECT(connection),
+                 "automatic-connection-events", FALSE, NULL);
+    g_object_unref(connection);
+    connection = NULL;
+    /* Free all IAP tracking resources */
+    g_slist_foreach(connections, g_mozilla_iap_free, NULL);
+    g_slist_free(connections);
+    connections = NULL;
+    /* Return to initial state */
+    connection_count = -1;
+#endif /* USE_CONIC */
 }
 
 gboolean
@@ -320,8 +472,8 @@ g_mozilla_connectivity_connect (gpointer data, gboolean reconnect)
     }
 #ifdef USE_CONIC
     connected = FALSE;
-    con_ic_connection_connect(connection, CON_IC_CONNECT_FLAG_NONE);
     is_connecting = TRUE;
+    con_ic_connection_connect(connection, CON_IC_CONNECT_FLAG_NONE);
 #endif
 
     if (gsloop) {
@@ -367,20 +519,19 @@ g_mozilla_parse_env_proxy (const gchar *env_name, const gchar *moz_proto)
     gchar c = 0;
     for (i = 0; i < env_len || i > SAFE_ENV_READ; i++) {
         c = env[i];
+        if (env[i] == '/' && env[i+1] == '/') {
+            k = 0; i++; port_parse = FALSE; continue;
+        }
         if (c == ':') {
-            if (env[i+1] != '/') {
-                port_parse = TRUE; server[k] = 0; k = 0; continue;
-            } else {
-                k = 0; continue;
-            }                
-        };
+            port_parse = TRUE; server[k] = 0; k = 0; continue;
+        }
         if (!port_parse && k < 20 && (c == '.' || g_ascii_isalnum(c)))
             server[k++] = c;
         else if (port_parse && k < 10 && g_ascii_isdigit(c))
             port[k++] = c;
         else break;
     }
-    port[k] = 0;
+    port[k<11?k:10] = 0;
     gint port_int = atoi((const char*)&port);
     gchar *moz_port = g_strdup_printf("network.proxy.%s_port", moz_proto);
     gchar *moz_server = g_strdup_printf("network.proxy.%s", moz_proto);
@@ -391,7 +542,7 @@ g_mozilla_parse_env_proxy (const gchar *env_name, const gchar *moz_proto)
     return TRUE;
 }
 
-void g_mozilla_connectivity_set_proxy ()
+gboolean g_mozilla_connectivity_set_proxy (void)
 {
     TRACE_LOG();
     gchar *proxytype = NULL, *autoconf = NULL;
@@ -518,4 +669,5 @@ void g_mozilla_connectivity_set_proxy ()
     EAL_IF_GFREE(proxytype);
     EAL_IF_GFREE(omit_proxy);
     TRACE_LOG();
+    return FALSE;
 }
