@@ -56,9 +56,19 @@ struct _GtkWebHistoryItem
 static void
 g_mozilla_history_list_delete_item (GMozillaHistoryList *self, gint pos)
 {
+    guint rv;
     TRACE_LOG();
     gchar * url = ((GtkMozHistoryItem*)G_MOZILLA_HISTORY_LIST(self)->items)[pos].url;
-    gtk_moz_embed_common_remove_history(url, 0);
+    if (!url)
+        return ;
+    rv = gtk_moz_embed_common_remove_history(url, 0);
+    /* we don't have NS_FAILED, this is NS_FAILED */
+    if (rv & 0x80000000) {
+        TRACE_LOG("gtk_moz_embed_common_remove_history failed");
+        return ;
+    }
+    ((GtkMozHistoryItem*)G_MOZILLA_HISTORY_LIST(self)->items)[pos].url = NULL;
+    ((GtkMozHistoryItem*)G_MOZILLA_HISTORY_LIST(self)->items)[pos].title = NULL;
     return ;
 }
 
