@@ -143,6 +143,45 @@ g_mozilla_plugin_list_get_item_type (GMozillaPluginList *self, gint pos)
     return NULL;
 }
 
+static guint
+g_mozilla_plugin_list_get_item_ctype (GMozillaPluginList *self, gint pos)
+{
+    TRACE_LOG();
+    PluginType pl_type = COMPONENT;
+    if (g_mozilla_plugin_list_position_ok(self, pos))
+    {
+        GtkMozPlugin *pl = g_list_nth_data((GList*)G_MOZILLA_PLUGIN_LIST(self)->items, pos);
+        if (pl) {
+            int p = 0;
+            if (!pl->isPlugin) {
+                char * buff = g_strdup_printf("%s/.mozilla/microb/extensions/%s", getenv("HOME"), pl->path);
+                if (buff) {
+                    p = access(buff, W_OK);
+                    g_free(buff);
+                }
+                if(p == -1)
+                  pl_type = SYS_COMPONENT;
+            } else
+                pl_type = PLUGIN;
+        }
+    }
+    return pl_type;
+}
+
+static guint
+g_mozilla_plugin_list_get_item_state (GMozillaPluginList *self, gint pos)
+{
+    TRACE_LOG();
+    if (g_mozilla_plugin_list_position_ok(self, pos))
+    {
+        //GtkMozPlugin *pl = g_list_nth_data((GList*)G_MOZILLA_PLUGIN_LIST(self)->items, pos);
+        //if (pl)
+        //    return pl->type;
+    }
+    return 0;
+}
+
+
 static void
 g_mozilla_plugin_list_interface_init (GWebPluginListIface *iface)
 {
@@ -161,6 +200,10 @@ g_mozilla_plugin_list_interface_init (GWebPluginListIface *iface)
                             g_mozilla_plugin_list_get_item_creator;
     iface->get_item_type   = (const gchar* (*) (GWebPluginList*, gint))
                              g_mozilla_plugin_list_get_item_type;
+    iface->get_item_ctype  = (guint (*) (GWebPluginList*, gint))
+                             g_mozilla_plugin_list_get_item_ctype;
+    iface->get_item_state  = (guint (*) (GWebPluginList*, gint))
+                             g_mozilla_plugin_list_get_item_state;
 }
 
 static void
