@@ -1146,6 +1146,25 @@ g_mozilla_web_get_pref (GMozillaWeb *self, gpointer type, gchar* name, gpointer
     return result;
 }
 
+static gboolean
+g_mozilla_web_set_glist_select_item (GMozillaWeb *self, gint select)
+{
+    self->glist_select_item = select;
+    return TRUE;
+}
+
+static gboolean
+g_mozilla_web_get_glist_item (GMozillaWeb *self, guint index, gchar** username)
+{
+    GList *list = self->username_list;
+    if(list == NULL)
+        return FALSE;
+    if(index > g_list_length(list))
+        return FALSE;
+    *username = g_strdup(g_list_nth_data(list, index));
+    return TRUE;
+}
+
 static GWebStatus
 g_mozilla_web_clear_all_history (GMozillaWeb *self)
 {
@@ -1270,6 +1289,11 @@ g_mozilla_web_interface_init (GWebIface *iface)
     iface->web_get_pref = (gboolean (*) (GWeb*, gpointer, const gchar*, gpointer))
         g_mozilla_web_get_pref;
 
+    iface->web_set_glist_select_item = (gboolean (*) (GWeb*, gint))
+        g_mozilla_web_set_glist_select_item;
+   iface->web_get_glist_item = (gboolean (*) (GWeb*, guint, gchar**))
+        g_mozilla_web_get_glist_item;
+
     iface->new_web_transfer_item              = (GWebTransferItem* (*) (GWeb*, const gchar*, const gchar*))
         g_mozilla_web_new_web_transfer_item;
     iface->initialize_web_transfer_item       = (GWebTransferItem* (*) (GWeb*, GObject*))
@@ -1441,6 +1465,8 @@ g_mozilla_web_instance_init (GTypeInstance *instance, gpointer  g_class)
     G_MOZILLA_WEB(instance)->global = NULL;
     G_MOZILLA_WEB(instance)->disabled_pllist = NULL;
     G_MOZILLA_WEB(instance)->mCertContexts = NULL;
+    G_MOZILLA_WEB(instance)->username_list = NULL;
+    G_MOZILLA_WEB(instance)->glist_select_item = 0;
     G_MOZILLA_WEB(instance)->anime_cursor = g_new0(AnimeCursor, 1);
     G_MOZILLA_WEB(instance)->default_cursor = gdk_cursor_new (GDK_LEFT_PTR);
 #ifdef MOZEAL_LOGGING
@@ -1832,6 +1858,13 @@ prompt_auth_cb (GtkMozPrompter * embed, GtkMozEmbed *mozembed, const gchar * dia
     // workaround for bug 46012                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 
 }
 
+void
+free_item(gpointer data, gpointer user_data)
+{
+    gchar *username = (gchar*) data;
+    g_free(username);
+}
+
 static gboolean
 select_cb (GtkMozPrompter * embed, GtkMozEmbed *mozembed, const gchar * dialog_title,
            const gchar * dialog_text, gpointer dialog_options,
@@ -1841,10 +1874,22 @@ select_cb (GtkMozPrompter * embed, GtkMozEmbed *mozembed, const gchar * dialog_t
     GMozillaEngine *self = g_object_get_data(G_OBJECT(mozembed), "eal_parent");
     g_return_val_if_fail (G_MOZILLA_ENGINE (self), FALSE);
 
+    GList *list = selfweb->username_list;
+    if(list) {
+        g_list_foreach(list, free_item, NULL);
+        g_list_free (list);
+        list = NULL;
+    }
+    GList *options = (GList*)dialog_options;
+    guint i = 0;
+    for(i = 0; i< g_list_length(options); i++)
+        list = g_list_append(list, g_strdup(g_list_nth_data(options, i)));
+    selfweb->username_list = list;
     gboolean ret_val;
     g_signal_emit_by_name (G_OBJECT (self), G_WEBWIDGET_SIGNAL_SELECT,
-                           dialog_title, dialog_text, dialog_options,
+                           dialog_title, dialog_text, NULL,
                            dialog_choice, &ret_val, NULL);
+    *(gint*)dialog_choice = selfweb->glist_select_item;
     return ret_val;
 }
 
